@@ -6,18 +6,17 @@ use warnings;
 use strict;
 
 # test environment at home: unpublished XML::Compile
-use lib '../XMLCompile/lib', '../../XMLCompile/lib';
+use lib '../XMLCompile/lib', '../LogReport/lib';
 
 package TestTools;
 use vars '$VERSION';
-$VERSION = '0.62';
+$VERSION = '0.63';
 use base 'Exporter';
 
 use XML::LibXML;
 use Test::More;
 use Test::Deep   qw/cmp_deeply/;
 
-use XML::Compile::Dumper;
 use POSIX        qw/_exit/;
 use Log::Report  qw/try/;
 use Data::Dumper qw/Dumper/;
